unit MultiCams_;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, MoDetect, StdCtrls, ComCtrls, Menus, MoDetect;

type
  TForm1 = class(TForm)
    MotionDetector3: TMotionDetector;
    MotionDetector4: TMotionDetector;
    GroupBox1: TGroupBox;
    MotionDetector1: TMotionDetector;
    CheckBox1: TCheckBox;
    Edit1: TEdit;
    Shape1: TShape;
    Image1: TImage;
    Memo1: TMemo;
    GroupBox2: TGroupBox;
    GroupBox3: TGroupBox;
    GroupBox4: TGroupBox;
    MotionDetector2: TMotionDetector;
    CheckBox2: TCheckBox;
    Shape2: TShape;
    Edit2: TEdit;
    Image2: TImage;
    Memo2: TMemo;
    CheckBox3: TCheckBox;
    Shape3: TShape;
    Edit3: TEdit;
    Image3: TImage;
    Memo3: TMemo;
    CheckBox4: TCheckBox;
    Shape4: TShape;
    Edit4: TEdit;
    Image4: TImage;
    Memo4: TMemo;
    VideoGrabber1: TVideoGrabber;
    VideoGrabber2: TVideoGrabber;
    VideoGrabber3: TVideoGrabber;
    VideoGrabber4: TVideoGrabber;
    GroupBox5: TGroupBox;
    CheckBox5: TCheckBox;
    Edit5: TEdit;
    UpDown1: TUpDown;
    Label1: TLabel;
    Edit6: TEdit;
    UpDown2: TUpDown;
    Label2: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure UpDown1Click(Sender: TObject; Button: TUDBtnType);
    procedure UpDown2Click(Sender: TObject; Button: TUDBtnType);
    procedure CheckBox5Click(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
    procedure CheckBox2Click(Sender: TObject);
    procedure CheckBox3Click(Sender: TObject);
    procedure CheckBox4Click(Sender: TObject);
    procedure MotionDetector1MotionDetected(Sender: TObject;
      GlobalMotionRatio: Double);
    procedure MotionDetector1MotionNotDetected(Sender: TObject);
    procedure MotionDetector2MotionDetected(Sender: TObject;
      GlobalMotionRatio: Double);
    procedure MotionDetector2MotionNotDetected(Sender: TObject);
    procedure MotionDetector3MotionDetected(Sender: TObject;
      GlobalMotionRatio: Double);
    procedure MotionDetector3MotionNotDetected(Sender: TObject);
    procedure MotionDetector4MotionDetected(Sender: TObject;
      GlobalMotionRatio: Double);
    procedure MotionDetector4MotionNotDetected(Sender: TObject);
  private
     procedure SetupGrids;
     procedure SetupGrid (MotionDetector: TMotionDetector; Memo: TMemo; GroupBox: TGroupBox);
     procedure SetupTVideoGrabber (VideoGrabber: TVideoGrabber; MotionDetector: TMotionDetector);
     procedure ProcessMotion (MotionDetected: boolean; MotionDetector: TMotionDetector; Shape: TShape; Memo: TMemo; Edit: TEdit; Image: TImage);
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}
//------------------------------------------------------------------------------

procedure TForm1.SetupGrid (MotionDetector: TMotionDetector; Memo: TMemo; GroupBox: TGroupBox);
var
   xCells, yCells: LongInt;
begin
   xCells := StrToIntDef (Edit5.Text, 8) ;
   yCells := StrToIntDef (Edit5.Text, 8) ;

   if MotionDetector.SetGridSize (xCells, yCells) then begin
      Memo.Text := MotionDetector.Get2DTextGrid;
      Memo.Width := MotionDetector.GridXCount * 14;
      Memo.Height := MotionDetector.GridYCount * 15 + 3;
   end;
   GroupBox.Width := Memo.Left + Memo.Width + 8; // we adapt the GroupBox size to the grid's size
   GroupBox.Height := Memo.Height + 24;

   if GroupBox.Height < 137 then begin
      GroupBox.Height := 137;                    // however we avoid having the GroupBox smaller than the video preview
   end;
end;
//------------------------------------------------------------------------------

procedure TForm1.SetupGrids;
begin
   SetupGrid (MotionDetector1, Memo1, GroupBox1);
   SetupGrid (MotionDetector2, Memo2, GroupBox2);
   SetupGrid (MotionDetector3, Memo3, GroupBox3);
   SetupGrid (MotionDetector4, Memo4, GroupBox4);
   GroupBox2.Top := GroupBox1.Top + GroupBox1.Height + 4;
   GroupBox3.Top := GroupBox2.Top + GroupBox2.Height + 4;
   GroupBox4.Top := GroupBox3.Top + GroupBox3.Height + 4;
   Form1.Width := GroupBox1.Left + GroupBox1.Width + 14;
   Form1.Height := GroupBox4.Top + GroupBox4.Height + 30;
end;
//------------------------------------------------------------------------------

procedure TForm1.SetupTVideoGrabber (VideoGrabber: TVideoGrabber; MotionDetector: TMotionDetector);
begin
   VideoGrabber.UseNearestVideoSize (160, 120, false); // we select the video size nearest to 160x120 (higher sizes increase CPU load)

   // THE 5 PROPERTIES BELOW CAN BE SET FROM THE OBJECT INSPECTOR

   VideoGrabber.MotionDetector := MotionDetector;  // connects the TMotionDetector component with the TVideoGrabber component
   VideoGrabber.AutoSize := False;                 // the video preview will be stretched to the component size
   VideoGrabber.BurstMode := True;                 // enabled frame capture
   VideoGrabber.BurstCount := 0;                   // not to stop frame capture
   VideoGrabber.BurstInterval := 0;                // not to skip frames
end;
//------------------------------------------------------------------------------

procedure TForm1.FormCreate(Sender: TObject);
begin
   SetupGrids;                        // let's setup the grid sizes...

   SetupTVideoGrabber (VideoGrabber1, MotionDetector1); // ... and the TVideoGrabber components
   SetupTVideoGrabber (VideoGrabber2, MotionDetector2);
   SetupTVideoGrabber (VideoGrabber3, MotionDetector3);
   SetupTVideoGrabber (VideoGrabber4, MotionDetector4);
end;
//------------------------------------------------------------------------------

procedure TForm1.UpDown1Click(Sender: TObject; Button: TUDBtnType);
begin
   SetupGrids; // let's resize the grids
end;
//------------------------------------------------------------------------------

procedure TForm1.UpDown2Click(Sender: TObject; Button: TUDBtnType);
begin
   MotionDetector1.SetGlobalSensitivity (StrToIntDef (Edit6.Text, 8));
   MotionDetector2.SetGlobalSensitivity (StrToIntDef (Edit6.Text, 8));
   MotionDetector3.SetGlobalSensitivity (StrToIntDef (Edit6.Text, 8));
   MotionDetector4.SetGlobalSensitivity (StrToIntDef (Edit6.Text, 8));
   SetupGrids; // to refresh the grid sensitivity display
end;
//------------------------------------------------------------------------------

procedure TForm1.CheckBox1Click(Sender: TObject);
begin
   if CheckBox1.Checked then begin
      VideoGrabber1.StartPreview;
   end
   else begin
      VideoGrabber1.StopPreview;
   end;
end;
//------------------------------------------------------------------------------

procedure TForm1.CheckBox2Click(Sender: TObject);
begin
   if CheckBox2.Checked then begin
      VideoGrabber2.StartPreview;
   end
   else begin
      VideoGrabber2.StopPreview;
   end;
end;
//------------------------------------------------------------------------------

procedure TForm1.CheckBox3Click(Sender: TObject);
begin
   if CheckBox3.Checked then begin
      VideoGrabber3.StartPreview;
   end
   else begin
      VideoGrabber3.StopPreview;
   end;
end;
//------------------------------------------------------------------------------

procedure TForm1.CheckBox4Click(Sender: TObject);
begin
   if CheckBox4.Checked then begin
      VideoGrabber4.StartPreview;
   end
   else begin
      VideoGrabber4.StopPreview;
   end;
end;
//------------------------------------------------------------------------------

procedure TForm1.CheckBox5Click(Sender: TObject);
begin
   MotionDetector1.ReduceVideoNoise := CheckBox5.Checked;
   MotionDetector2.ReduceVideoNoise := CheckBox5.Checked;
   MotionDetector3.ReduceVideoNoise := CheckBox5.Checked;
   MotionDetector4.ReduceVideoNoise := CheckBox5.Checked;
end;
//------------------------------------------------------------------------------

procedure TForm1.ProcessMotion (MotionDetected: boolean; MotionDetector: TMotionDetector; Shape: TShape; Memo: TMemo; Edit: TEdit; Image: TImage);
begin
   if MotionDetected then begin
      Shape.Brush.Color := clLime;
      Memo.Text := MotionDetector.Get2DTextMotion;
      Edit.Text := FloatToStr (MotionDetector.GlobalMotionRatio);
      Image.Picture.Bitmap.Assign (MotionDetector.LastFrame);
   end
   else begin
      Shape.Brush.Color := clGreen;
      Memo.Text := '';
      Edit.Text := '';
   end;
end;
//------------------------------------------------------------------------------

procedure TForm1.MotionDetector1MotionDetected(Sender: TObject; GlobalMotionRatio: Double);
begin
   ProcessMotion (true, MotionDetector1, Shape1, Memo1, Edit1, Image1);
end;
//------------------------------------------------------------------------------

procedure TForm1.MotionDetector2MotionDetected(Sender: TObject; GlobalMotionRatio: Double);
begin
   ProcessMotion (true, MotionDetector2, Shape2, Memo2, Edit2, Image2);
end;
//------------------------------------------------------------------------------

procedure TForm1.MotionDetector3MotionDetected(Sender: TObject; GlobalMotionRatio: Double);
begin
   ProcessMotion (true, MotionDetector3, Shape3, Memo3, Edit3, Image3);
end;
//------------------------------------------------------------------------------

procedure TForm1.MotionDetector4MotionDetected(Sender: TObject; GlobalMotionRatio: Double);
begin
   ProcessMotion (true, MotionDetector4, Shape4, Memo4, Edit4, Image4);
end;
//------------------------------------------------------------------------------

procedure TForm1.MotionDetector1MotionNotDetected(Sender: TObject);
begin
   ProcessMotion (false, MotionDetector1, Shape1, Memo1, Edit1, Image1);
end;
//------------------------------------------------------------------------------

procedure TForm1.MotionDetector2MotionNotDetected(Sender: TObject);
begin
   ProcessMotion (false, MotionDetector2, Shape2, Memo2, Edit2, Image2);
end;
//------------------------------------------------------------------------------

procedure TForm1.MotionDetector3MotionNotDetected(Sender: TObject);
begin
   ProcessMotion (false, MotionDetector3, Shape3, Memo3, Edit3, Image3);
end;
//------------------------------------------------------------------------------

procedure TForm1.MotionDetector4MotionNotDetected(Sender: TObject);
begin
   ProcessMotion (false, MotionDetector4, Shape4, Memo4, Edit4, Image4);
end;
//------------------------------------------------------------------------------

end.
